var fs = require('fs');
var mkdirp = require("mkdirp");
var path = require('path');
var consolidate = require('consolidate');
var inquirer = require("inquirer");
var Q = require('q');
var _ = require('underscore');

function readFile(path, encoding) {
	var deferred = Q.defer();
	fs.readFile(path, encoding || 'utf8', function(err, content) {
		if(err) {
			deferred.reject(err);
		} else {
			deferred.resolve(content);
		}
	});
	return deferred.promise;
}

function writeFile(filepath, encoding, contents) {
	var deferred = Q.defer();
    mkdirp(path.dirname(filepath), function (err) {
		if(err) return deferred.reject(err);
        fs.writeFile(filepath, contents, encoding, function(err) {
			if(err) {
				deferred.reject(err);
			} else {
				deferred.resolve();
			}
		});
    });
	
	return deferred.promise;
}

function setOptions(v) {
	var v = _.extend({
			encoding: 'utf8',
			engine: 'lodash',
	}, v);
	return Q.all(v);
}

function beforePrompts(v) {
	if(v.beforePrompt) {
		var deferred = Q.defer();
		v.beforePrompt(v, function(err) {
			if(err) {
				deferred.reject(err);
			} else {
				deferred.resolve(v);
			}
		});
		return deferred.promise;
	} else {
		return Q.all(v);
	}	
}

function askPrompts(v) {
	var deferred = Q.defer();
	inquirer.prompt(v.prompts || [], function (inputs) {
		v.inputs = inputs;
		deferred.resolve(v);
	});
	return deferred.promise;
}

function buildViewModel(v) {
	var viewModel = v.inputs;
	viewModel.context = v.context || {};
	v.viewModel = viewModel;
	return Q.all(v);
}

function beforeRender(v) {
	if(v.beforeRender) {
		var deferred = Q.defer();
		v.beforeRender(v, function(err) {
			if(err) {
				deferred.reject(err);
			} else {
				deferred.resolve(v);
			}
		});
		return deferred.promise;
	} else {
		return Q.all(v);
	}	
}

function render(engineName, templateContent, viewModel) {
	var deferred = Q.defer();
	var engine = consolidate[engineName];
	if(!engine) {
		return deferred.reject('Consolidate engine not installed: ' + engineName);
	}
	engine.render(templateContent, viewModel, function (err, renderedContent) {
		if(err) {
			deferred.reject(err);
		} else {
			deferred.resolve(renderedContent);
		}
	});
	
	return deferred.promise;
}

function renderFileNames(v) {
	return Q.all(_.map(v.translate, function(fileNameTemplate, key) {
		return render(v.engine, fileNameTemplate, v.viewModel)
			.then(function(fileName) {
				v.translate[key] = fileName;
			});
	}))
	.then(function() {
		return v;
	});;
}

function renderFileContent(v) {
	return Q.all(_.map(v.translate, function(renderedFileName, templateFileName) {
		templateFileName = path.join(v.paths.templateDir, templateFileName);
		
		return readFile(templateFileName, v.encoding)
			.then(function(content) {
				return render(v.engine, content, v.viewModel);
			})
			.then(function(content) {
				return writeFile(renderedFileName, v.encoding, content);
			})
			.then(function() {
				if(v.file) {
					v.file(templateFileName, renderedFileName, v.viewModel);
				}
			});
	}))
	.then(function() {
		return v;
	});
}

function afterPrompts(v) {
	if(v.after) {
		v.after();
	}
	return Q.all(v);
}

module.exports = function (options) {
	return setOptions(options)
		.then(beforePrompts)
		.then(askPrompts)
		.then(buildViewModel)
		.then(beforeRender)
		.then(renderFileNames)
		.then(renderFileContent)
		.then(afterPrompts)
		.done();
}